import unittest
from imputegap.recovery.benchmark import Benchmark

class TestBenchmarking(unittest.TestCase):

    def test_benchmarking(self):
        """
        the goal is to test if only the simple imputation with ST-MVL has the expected outcome
        """
        bench = Benchmark()
        expected_datasets = ["eeg-alcohol"]

        opti_bayesian = {"optimizer": "bayesian", "options": {"n_calls": 2, "n_random_starts": 50, "acq_func": "gp_hedge", "metrics": "RMSE"}}
        optimizers = [opti_bayesian]

        algorithms_full = ["mean", "cdrec", "stmvl", "iim", "mrnn"]
        patterns_small = ["mcar"]

        x_axis = [0.05, 0.1, 0.2, 0.4, 0.6, 0.8]

        bench.eval(datasets=expected_datasets, optimizers=optimizers, algorithms=algorithms_full, patterns=patterns_small, x_axis=x_axis, runs=-1)
        results_benchmarking = bench.list_results
        results_benchmarking = results_benchmarking[0]
        expected_datasets = ["eegalcohol"]

        # Check that all datasets exist
        actual_datasets = list(results_benchmarking.keys())
        self.assertCountEqual(
            actual_datasets, expected_datasets,
            f"Missing datasets. Expected: {expected_datasets}, Found: {actual_datasets}"
        )

        # For each dataset, validate structure and values
        for dataset, dataset_data in results_benchmarking.items():
            if not dataset_data:  # If dataset is empty, skip validation
                continue

            # Check that patterns exist (e.g., 'mcar')
            self.assertIn(
                "mcar", dataset_data,
                f"Dataset '{dataset}' is missing 'mcar' scenario."
            )
            scenario_data = dataset_data["mcar"]

            # Check that all algorithms exist
            expected_algorithms = {"stmvl", "cdrec", "iim", "mean", "mrnn"}
            actual_algorithms = set(scenario_data.keys())
            self.assertTrue(
                expected_algorithms.issubset(actual_algorithms),
                f"Missing algorithms in dataset '{dataset}'. Expected: {expected_algorithms}, Found: {actual_algorithms}"
            )

            # Check that each algorithm contains the expected keys
            expected_keys = {"0.05", "0.1", "0.2", "0.4", "0.8"}
            for algorithm, algorithm_data in scenario_data.items():
                for key in expected_keys:
                    self.assertIn(
                        key, algorithm_data.get("bayesian", {}),
                        f"Algorithm '{algorithm}' in dataset '{dataset}' is missing key '{key}'."
                    )

                    sub_data = algorithm_data["bayesian"].get(key, {})
                    for score_key, score_value in sub_data.get("scores", {}).items():
                        self.assertIsInstance(
                            score_value,
                            (float, int),  # Correct usage
                            f"Score '{score_key}' in dataset '{dataset}', algorithm '{algorithm}', key '{key}' is not a float or int."
                        )
                    for time_key, time_value in sub_data.get("times", {}).items():
                        self.assertIsInstance(
                            time_value,
                            (float, int),  # Correct usage
                            f"Time '{time_key}' in dataset '{dataset}', algorithm '{algorithm}', key '{key}'is not a float."
                        )

    def test_benchmarking_matrix(self):
        """
        the goal is to test if only the simple imputation with ST-MVL has the expected outcome
        """
        alpha_1 = {"eeg-alcohol":  {"mcar":
                    {"mean": {"bayesian": { "0.05": {"scores": {"RMSE": 0, "MAE": 1, "MI": 2, "CORRELATION": 3}, "times": {"contamination": 4, "optimization": 5, "imputation": 6}}, "0.2": {"scores": {"RMSE": 1, "MAE": 2, "MI": 3, "CORRELATION": 4}, "times": {"contamination": 5, "optimization": 6, "imputation": 7}}, "0.4": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.6": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.8": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}}},
                    "cdrec": {"bayesian": { "0.05": {"scores": {"RMSE": -10, "MAE": 1, "MI": 2, "CORRELATION": 3}, "times": {"contamination": 4, "optimization": 5, "imputation": 6}}, "0.2": {"scores": {"RMSE": 1, "MAE": 2, "MI": 3, "CORRELATION": 4}, "times": {"contamination": 5, "optimization": 6, "imputation": 7}}, "0.4": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.6": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.8": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}}},
                    "stmvl": {"bayesian": { "0.05": {"scores": {"RMSE": -40, "MAE": 1, "MI": 2, "CORRELATION": 3}, "times": {"contamination": 4, "optimization": 5, "imputation": 6}}, "0.2": {"scores": {"RMSE": 1, "MAE": 2, "MI": 3, "CORRELATION": 4}, "times": {"contamination": 5, "optimization": 6, "imputation": 7}}, "0.4": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.6": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.8": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}}},
                    "mrnn": {"bayesian": { "0.05": {"scores": {"RMSE": -30, "MAE": 1, "MI": 2, "CORRELATION": 3}, "times": {"contamination": 4, "optimization": 5, "imputation": 6}}, "0.2": {"scores": {"RMSE": 1, "MAE": 2, "MI": 3, "CORRELATION": 4}, "times": {"contamination": 5, "optimization": 6, "imputation": 7}}, "0.4": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.6": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.8": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}}},
                    "iim": {"bayesian": { "0.05": {"scores": {"RMSE": 40, "MAE": 1, "MI": 2, "CORRELATION": 3}, "times": {"contamination": 4, "optimization": 5, "imputation": 6}}, "0.2": {"scores": {"RMSE": 1, "MAE": 2, "MI": 3, "CORRELATION": 4}, "times": {"contamination": 5, "optimization": 6, "imputation": 7}}, "0.4": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.6": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.8": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}}},
        }}}

        alpha_2 = {"eeg-alcohol":  {"mcar":
                    {"mean": {"bayesian": { "0.05": {"scores": {"RMSE": -1, "MAE": 1, "MI": 2, "CORRELATION": 3}, "times": {"contamination": 4, "optimization": 5, "imputation": 6}}, "0.2": {"scores": {"RMSE": 1, "MAE": 2, "MI": 3, "CORRELATION": 4}, "times": {"contamination": 5, "optimization": 6, "imputation": 7}}, "0.4": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.6": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.8": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}}},
                    "cdrec": {"bayesian": { "0.05": {"scores": {"RMSE": 50, "MAE": 1, "MI": 2, "CORRELATION": 3}, "times": {"contamination": 4, "optimization": 5, "imputation": 6}}, "0.2": {"scores": {"RMSE": 1, "MAE": 2, "MI": 3, "CORRELATION": 4}, "times": {"contamination": 5, "optimization": 6, "imputation": 7}}, "0.4": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.6": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.8": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}}},
                    "stmvl": {"bayesian": { "0.05": {"scores": {"RMSE": 20, "MAE": 1, "MI": 2, "CORRELATION": 3}, "times": {"contamination": 4, "optimization": 5, "imputation": 6}}, "0.2": {"scores": {"RMSE": 1, "MAE": 2, "MI": 3, "CORRELATION": 4}, "times": {"contamination": 5, "optimization": 6, "imputation": 7}}, "0.4": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.6": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.8": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}}},
                    "mrnn": {"bayesian": { "0.05": {"scores": {"RMSE": 30, "MAE": 1, "MI": 2, "CORRELATION": 3}, "times": {"contamination": 4, "optimization": 5, "imputation": 6}}, "0.2": {"scores": {"RMSE": 1, "MAE": 2, "MI": 3, "CORRELATION": 4}, "times": {"contamination": 5, "optimization": 6, "imputation": 7}}, "0.4": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.6": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.8": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}}},
                    "iim": {"bayesian": { "0.05": {"scores": {"RMSE": -100, "MAE": 1, "MI": 2, "CORRELATION": 3}, "times": {"contamination": 4, "optimization": 5, "imputation": 6}}, "0.2": {"scores": {"RMSE": 1, "MAE": 2, "MI": 3, "CORRELATION": 4}, "times": {"contamination": 5, "optimization": 6, "imputation": 7}}, "0.4": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.6": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.8": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}}},
        }}}

        beta_1 = {"eeg-reading":  {"mcar":
                    {"mean": {"bayesian": { "0.05": {"scores": {"RMSE": 0, "MAE": 1, "MI": 2, "CORRELATION": 3}, "times": {"contamination": 4, "optimization": 5, "imputation": 6}}, "0.2": {"scores": {"RMSE": 1, "MAE": 2, "MI": 3, "CORRELATION": 4}, "times": {"contamination": 5, "optimization": 6, "imputation": 7}}, "0.4": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.6": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.8": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}}},
                    "cdrec": {"bayesian": { "0.05": {"scores": {"RMSE": -10, "MAE": 1, "MI": 2, "CORRELATION": 3}, "times": {"contamination": 4, "optimization": 5, "imputation": 6}}, "0.2": {"scores": {"RMSE": 1, "MAE": 2, "MI": 3, "CORRELATION": 4}, "times": {"contamination": 5, "optimization": 6, "imputation": 7}}, "0.4": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.6": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.8": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}}},
                    "stmvl": {"bayesian": { "0.05": {"scores": {"RMSE": -40, "MAE": 1, "MI": 2, "CORRELATION": 3}, "times": {"contamination": 4, "optimization": 5, "imputation": 6}}, "0.2": {"scores": {"RMSE": 1, "MAE": 2, "MI": 3, "CORRELATION": 4}, "times": {"contamination": 5, "optimization": 6, "imputation": 7}}, "0.4": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.6": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.8": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}}},
                    "mrnn": {"bayesian": { "0.05": {"scores": {"RMSE": -30, "MAE": 1, "MI": 2, "CORRELATION": 3}, "times": {"contamination": 4, "optimization": 5, "imputation": 6}}, "0.2": {"scores": {"RMSE": 1, "MAE": 2, "MI": 3, "CORRELATION": 4}, "times": {"contamination": 5, "optimization": 6, "imputation": 7}}, "0.4": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.6": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.8": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}}},
                    "iim": {"bayesian": { "0.05": {"scores": {"RMSE": 40, "MAE": 1, "MI": 2, "CORRELATION": 3}, "times": {"contamination": 4, "optimization": 5, "imputation": 6}}, "0.2": {"scores": {"RMSE": 1, "MAE": 2, "MI": 3, "CORRELATION": 4}, "times": {"contamination": 5, "optimization": 6, "imputation": 7}}, "0.4": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.6": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.8": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}}},
        }}}

        beta_2 = {"eeg-reading":  {"mcar":
                    {"mean": {"bayesian": { "0.05": {"scores": {"RMSE": -1, "MAE": 1, "MI": 2, "CORRELATION": 3}, "times": {"contamination": 4, "optimization": 5, "imputation": 6}}, "0.2": {"scores": {"RMSE": 1, "MAE": 2, "MI": 3, "CORRELATION": 4}, "times": {"contamination": 5, "optimization": 6, "imputation": 7}}, "0.4": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.6": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.8": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}}},
                    "cdrec": {"bayesian": { "0.05": {"scores": {"RMSE": 50, "MAE": 1, "MI": 2, "CORRELATION": 3}, "times": {"contamination": 4, "optimization": 5, "imputation": 6}}, "0.2": {"scores": {"RMSE": 1, "MAE": 2, "MI": 3, "CORRELATION": 4}, "times": {"contamination": 5, "optimization": 6, "imputation": 7}}, "0.4": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.6": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.8": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}}},
                    "stmvl": {"bayesian": { "0.05": {"scores": {"RMSE": 20, "MAE": 1, "MI": 2, "CORRELATION": 3}, "times": {"contamination": 4, "optimization": 5, "imputation": 6}}, "0.2": {"scores": {"RMSE": 1, "MAE": 2, "MI": 3, "CORRELATION": 4}, "times": {"contamination": 5, "optimization": 6, "imputation": 7}}, "0.4": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.6": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.8": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}}},
                    "mrnn": {"bayesian": { "0.05": {"scores": {"RMSE": 30, "MAE": 1, "MI": 2, "CORRELATION": 3}, "times": {"contamination": 4, "optimization": 5, "imputation": 6}}, "0.2": {"scores": {"RMSE": 1, "MAE": 2, "MI": 3, "CORRELATION": 4}, "times": {"contamination": 5, "optimization": 6, "imputation": 7}}, "0.4": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.6": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.8": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}}},
                    "iim": {"bayesian": { "0.05": {"scores": {"RMSE": -100, "MAE": 1, "MI": 2, "CORRELATION": 3}, "times": {"contamination": 4, "optimization": 5, "imputation": 6}}, "0.2": {"scores": {"RMSE": 1, "MAE": 2, "MI": 3, "CORRELATION": 4}, "times": {"contamination": 5, "optimization": 6, "imputation": 7}}, "0.4": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.6": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.8": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}}},
        }}}

        delta_1 = {"fmri-objectviewing":  {"mcar":
                    {"mean": {"bayesian": { "0.05": {"scores": {"RMSE": 0, "MAE": 1, "MI": 2, "CORRELATION": 3}, "times": {"contamination": 4, "optimization": 5, "imputation": 6}}, "0.2": {"scores": {"RMSE": 1, "MAE": 2, "MI": 3, "CORRELATION": 4}, "times": {"contamination": 5, "optimization": 6, "imputation": 7}}, "0.4": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.6": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.8": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}}},
                    "cdrec": {"bayesian": { "0.05": {"scores": {"RMSE": -10, "MAE": 1, "MI": 2, "CORRELATION": 3}, "times": {"contamination": 4, "optimization": 5, "imputation": 6}}, "0.2": {"scores": {"RMSE": 1, "MAE": 2, "MI": 3, "CORRELATION": 4}, "times": {"contamination": 5, "optimization": 6, "imputation": 7}}, "0.4": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.6": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.8": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}}},
                    "stmvl": {"bayesian": { "0.05": {"scores": {"RMSE": -40, "MAE": 1, "MI": 2, "CORRELATION": 3}, "times": {"contamination": 4, "optimization": 5, "imputation": 6}}, "0.2": {"scores": {"RMSE": 1, "MAE": 2, "MI": 3, "CORRELATION": 4}, "times": {"contamination": 5, "optimization": 6, "imputation": 7}}, "0.4": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.6": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.8": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}}},
                    "mrnn": {"bayesian": { "0.05": {"scores": {"RMSE": -30, "MAE": 1, "MI": 2, "CORRELATION": 3}, "times": {"contamination": 4, "optimization": 5, "imputation": 6}}, "0.2": {"scores": {"RMSE": 1, "MAE": 2, "MI": 3, "CORRELATION": 4}, "times": {"contamination": 5, "optimization": 6, "imputation": 7}}, "0.4": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.6": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.8": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}}},
                    "iim": {"bayesian": { "0.05": {"scores": {"RMSE": 40, "MAE": 1, "MI": 2, "CORRELATION": 3}, "times": {"contamination": 4, "optimization": 5, "imputation": 6}}, "0.2": {"scores": {"RMSE": 1, "MAE": 2, "MI": 3, "CORRELATION": 4}, "times": {"contamination": 5, "optimization": 6, "imputation": 7}}, "0.4": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.6": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.8": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}}},
        }}}

        delta_2 = {"fmri-objectviewing":  {"mcar":
                    {"mean": {"bayesian": { "0.05": {"scores": {"RMSE": -1, "MAE": 1, "MI": 2, "CORRELATION": 3}, "times": {"contamination": 4, "optimization": 5, "imputation": 6}}, "0.2": {"scores": {"RMSE": 1, "MAE": 2, "MI": 3, "CORRELATION": 4}, "times": {"contamination": 5, "optimization": 6, "imputation": 7}}, "0.4": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.6": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.8": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}}},
                    "cdrec": {"bayesian": { "0.05": {"scores": {"RMSE": 50, "MAE": 1, "MI": 2, "CORRELATION": 3}, "times": {"contamination": 4, "optimization": 5, "imputation": 6}}, "0.2": {"scores": {"RMSE": 1, "MAE": 2, "MI": 3, "CORRELATION": 4}, "times": {"contamination": 5, "optimization": 6, "imputation": 7}}, "0.4": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.6": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.8": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}}},
                    "stmvl": {"bayesian": { "0.05": {"scores": {"RMSE": 20, "MAE": 1, "MI": 2, "CORRELATION": 3}, "times": {"contamination": 4, "optimization": 5, "imputation": 6}}, "0.2": {"scores": {"RMSE": 1, "MAE": 2, "MI": 3, "CORRELATION": 4}, "times": {"contamination": 5, "optimization": 6, "imputation": 7}}, "0.4": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.6": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.8": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}}},
                    "mrnn": {"bayesian": { "0.05": {"scores": {"RMSE": 30, "MAE": 1, "MI": 2, "CORRELATION": 3}, "times": {"contamination": 4, "optimization": 5, "imputation": 6}}, "0.2": {"scores": {"RMSE": 1, "MAE": 2, "MI": 3, "CORRELATION": 4}, "times": {"contamination": 5, "optimization": 6, "imputation": 7}}, "0.4": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.6": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.8": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}}},
                    "iim": {"bayesian": { "0.05": {"scores": {"RMSE": -100, "MAE": 1, "MI": 2, "CORRELATION": 3}, "times": {"contamination": 4, "optimization": 5, "imputation": 6}}, "0.2": {"scores": {"RMSE": 1, "MAE": 2, "MI": 3, "CORRELATION": 4}, "times": {"contamination": 5, "optimization": 6, "imputation": 7}}, "0.4": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.6": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.8": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}}},
        }}}

        epsilon_1 = {"chlorine":  {"mcar":
                    {"mean": {"bayesian": { "0.05": {"scores": {"RMSE": 0, "MAE": 1, "MI": 2, "CORRELATION": 3}, "times": {"contamination": 4, "optimization": 5, "imputation": 6}}, "0.2": {"scores": {"RMSE": 1, "MAE": 2, "MI": 3, "CORRELATION": 4}, "times": {"contamination": 5, "optimization": 6, "imputation": 7}}, "0.4": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.6": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.8": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}}},
                    "cdrec": {"bayesian": { "0.05": {"scores": {"RMSE": -10, "MAE": 1, "MI": 2, "CORRELATION": 3}, "times": {"contamination": 4, "optimization": 5, "imputation": 6}}, "0.2": {"scores": {"RMSE": 1, "MAE": 2, "MI": 3, "CORRELATION": 4}, "times": {"contamination": 5, "optimization": 6, "imputation": 7}}, "0.4": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.6": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.8": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}}},
                    "stmvl": {"bayesian": { "0.05": {"scores": {"RMSE": -40, "MAE": 1, "MI": 2, "CORRELATION": 3}, "times": {"contamination": 4, "optimization": 5, "imputation": 6}}, "0.2": {"scores": {"RMSE": 1, "MAE": 2, "MI": 3, "CORRELATION": 4}, "times": {"contamination": 5, "optimization": 6, "imputation": 7}}, "0.4": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.6": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.8": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}}},
                    "mrnn": {"bayesian": { "0.05": {"scores": {"RMSE": -30, "MAE": 1, "MI": 2, "CORRELATION": 3}, "times": {"contamination": 4, "optimization": 5, "imputation": 6}}, "0.2": {"scores": {"RMSE": 1, "MAE": 2, "MI": 3, "CORRELATION": 4}, "times": {"contamination": 5, "optimization": 6, "imputation": 7}}, "0.4": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.6": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.8": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}}},
                    "iim": {"bayesian": { "0.05": {"scores": {"RMSE": 40, "MAE": 1, "MI": 2, "CORRELATION": 3}, "times": {"contamination": 4, "optimization": 5, "imputation": 6}}, "0.2": {"scores": {"RMSE": 1, "MAE": 2, "MI": 3, "CORRELATION": 4}, "times": {"contamination": 5, "optimization": 6, "imputation": 7}}, "0.4": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.6": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.8": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}}},
        }}}

        epsilon_2 = {"chlorine":  {"mcar":
                    {"mean": {"bayesian": { "0.05": {"scores": {"RMSE": -1, "MAE": 1, "MI": 2, "CORRELATION": 3}, "times": {"contamination": 4, "optimization": 5, "imputation": 6}}, "0.2": {"scores": {"RMSE": 1, "MAE": 2, "MI": 3, "CORRELATION": 4}, "times": {"contamination": 5, "optimization": 6, "imputation": 7}}, "0.4": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.6": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.8": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}}},
                    "cdrec": {"bayesian": { "0.05": {"scores": {"RMSE": 50, "MAE": 1, "MI": 2, "CORRELATION": 3}, "times": {"contamination": 4, "optimization": 5, "imputation": 6}}, "0.2": {"scores": {"RMSE": 1, "MAE": 2, "MI": 3, "CORRELATION": 4}, "times": {"contamination": 5, "optimization": 6, "imputation": 7}}, "0.4": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.6": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.8": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}}},
                    "stmvl": {"bayesian": { "0.05": {"scores": {"RMSE": 20, "MAE": 1, "MI": 2, "CORRELATION": 3}, "times": {"contamination": 4, "optimization": 5, "imputation": 6}}, "0.2": {"scores": {"RMSE": 1, "MAE": 2, "MI": 3, "CORRELATION": 4}, "times": {"contamination": 5, "optimization": 6, "imputation": 7}}, "0.4": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.6": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.8": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}}},
                    "mrnn": {"bayesian": { "0.05": {"scores": {"RMSE": 30, "MAE": 1, "MI": 2, "CORRELATION": 3}, "times": {"contamination": 4, "optimization": 5, "imputation": 6}}, "0.2": {"scores": {"RMSE": 1, "MAE": 2, "MI": 3, "CORRELATION": 4}, "times": {"contamination": 5, "optimization": 6, "imputation": 7}}, "0.4": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.6": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.8": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}}},
                    "iim": {"bayesian": { "0.05": {"scores": {"RMSE": -100, "MAE": 1, "MI": 2, "CORRELATION": 3}, "times": {"contamination": 4, "optimization": 5, "imputation": 6}}, "0.2": {"scores": {"RMSE": 1, "MAE": 2, "MI": 3, "CORRELATION": 4}, "times": {"contamination": 5, "optimization": 6, "imputation": 7}}, "0.4": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.6": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.8": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}}},
        }}}

        gamma_1 = {"drift":  {"mcar":
                    {"mean": {"bayesian": { "0.05": {"scores": {"RMSE": 0, "MAE": 1, "MI": 2, "CORRELATION": 3}, "times": {"contamination": 4, "optimization": 5, "imputation": 6}}, "0.2": {"scores": {"RMSE": 1, "MAE": 2, "MI": 3, "CORRELATION": 4}, "times": {"contamination": 5, "optimization": 6, "imputation": 7}}, "0.4": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.6": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.8": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}}},
                    "cdrec": {"bayesian": { "0.05": {"scores": {"RMSE": -10, "MAE": 1, "MI": 2, "CORRELATION": 3}, "times": {"contamination": 4, "optimization": 5, "imputation": 6}}, "0.2": {"scores": {"RMSE": 1, "MAE": 2, "MI": 3, "CORRELATION": 4}, "times": {"contamination": 5, "optimization": 6, "imputation": 7}}, "0.4": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.6": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.8": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}}},
                    "stmvl": {"bayesian": { "0.05": {"scores": {"RMSE": -40, "MAE": 1, "MI": 2, "CORRELATION": 3}, "times": {"contamination": 4, "optimization": 5, "imputation": 6}}, "0.2": {"scores": {"RMSE": 1, "MAE": 2, "MI": 3, "CORRELATION": 4}, "times": {"contamination": 5, "optimization": 6, "imputation": 7}}, "0.4": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.6": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.8": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}}},
                    "mrnn": {"bayesian": { "0.05": {"scores": {"RMSE": -30, "MAE": 1, "MI": 2, "CORRELATION": 3}, "times": {"contamination": 4, "optimization": 5, "imputation": 6}}, "0.2": {"scores": {"RMSE": 1, "MAE": 2, "MI": 3, "CORRELATION": 4}, "times": {"contamination": 5, "optimization": 6, "imputation": 7}}, "0.4": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.6": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.8": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}}},
                    "iim": {"bayesian": { "0.05": {"scores": {"RMSE": 40, "MAE": 1, "MI": 2, "CORRELATION": 3}, "times": {"contamination": 4, "optimization": 5, "imputation": 6}}, "0.2": {"scores": {"RMSE": 1, "MAE": 2, "MI": 3, "CORRELATION": 4}, "times": {"contamination": 5, "optimization": 6, "imputation": 7}}, "0.4": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.6": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.8": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}}},
        }}}

        gamma_2 = {"drift":  {"mcar":
                    {"mean": {"bayesian": { "0.05": {"scores": {"RMSE": -1, "MAE": 1, "MI": 2, "CORRELATION": 3}, "times": {"contamination": 4, "optimization": 5, "imputation": 6}}, "0.2": {"scores": {"RMSE": 1, "MAE": 2, "MI": 3, "CORRELATION": 4}, "times": {"contamination": 5, "optimization": 6, "imputation": 7}}, "0.4": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.6": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.8": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}}},
                    "cdrec": {"bayesian": { "0.05": {"scores": {"RMSE": 50, "MAE": 1, "MI": 2, "CORRELATION": 3}, "times": {"contamination": 4, "optimization": 5, "imputation": 6}}, "0.2": {"scores": {"RMSE": 1, "MAE": 2, "MI": 3, "CORRELATION": 4}, "times": {"contamination": 5, "optimization": 6, "imputation": 7}}, "0.4": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.6": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.8": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}}},
                    "stmvl": {"bayesian": { "0.05": {"scores": {"RMSE": 20, "MAE": 1, "MI": 2, "CORRELATION": 3}, "times": {"contamination": 4, "optimization": 5, "imputation": 6}}, "0.2": {"scores": {"RMSE": 1, "MAE": 2, "MI": 3, "CORRELATION": 4}, "times": {"contamination": 5, "optimization": 6, "imputation": 7}}, "0.4": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.6": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.8": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}}},
                    "mrnn": {"bayesian": { "0.05": {"scores": {"RMSE": 30, "MAE": 1, "MI": 2, "CORRELATION": 3}, "times": {"contamination": 4, "optimization": 5, "imputation": 6}}, "0.2": {"scores": {"RMSE": 1, "MAE": 2, "MI": 3, "CORRELATION": 4}, "times": {"contamination": 5, "optimization": 6, "imputation": 7}}, "0.4": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.6": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.8": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}}},
                    "iim": {"bayesian": { "0.05": {"scores": {"RMSE": -100, "MAE": 1, "MI": 2, "CORRELATION": 3}, "times": {"contamination": 4, "optimization": 5, "imputation": 6}}, "0.2": {"scores": {"RMSE": 1, "MAE": 2, "MI": 3, "CORRELATION": 4}, "times": {"contamination": 5, "optimization": 6, "imputation": 7}}, "0.4": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.6": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}, "0.8": {"scores": {"RMSE": 0.5, "MAE": 1.5, "MI": 2.5, "CORRELATION": 3.5}, "times": {"contamination": 4.5, "optimization": 5.5, "imputation": 6.5}}}},
        }}}

        bench = Benchmark()
        scores_list, algos, sets = bench.avg_results(alpha_1, alpha_2, beta_1, beta_2, delta_1, delta_2, epsilon_1, epsilon_2, gamma_1, gamma_2)

        print(scores_list)

        # Check that algos contains all expected algorithms
        expected_algos = ["stmvl", "cdrec", "iim", "mean", "mrnn"]
        self.assertCountEqual(
            algos, expected_algos,
            f"Missing algorithms in Benchmarking. Expected: {expected_algos}, Found: {algos}"
        )

        # Check that sets contains all expected datasets
        expected_sets = ["eeg-alcohol", "eeg-reading", "fmri-objectviewing", "chlorine", "drift"]
        self.assertCountEqual(
            sets, expected_sets,
            f"Missing datasets in Benchmarking. Expected: {expected_sets}, Found: {sets}"
        )

        # Check average values for each dataset and algorithm
        algo_order = ['cdrec', 'iim', 'mean', 'mrnn', 'stmvl']
        expected_rmse = {
            "cdrec": 4.5,
            "iim": -5.5,
            "mean": 0.4,
            "mrnn": 0.5,
            "stmvl": -1.5
        }

        # Iterate through the matrix
        for i, dataset_scores in enumerate(scores_list):
            for j, rmse_value in enumerate(dataset_scores):
                algo = algo_order[j]  # Match the algorithm based on column index
                expected_value = expected_rmse[algo]
                self.assertEqual(
                    rmse_value,
                    expected_value,
                    f"Unexpected RMSE for algorithm '{algo}' at dataset index {i}."
                )


        validation = bench.generate_heatmap(scores_list, algos, sets, metric="RMSE", save_dir="./reports", display=False)

        self.assertTrue(validation)
